
#include <stdio.h>
#include <string.h>

#include "asocia.h"
#include "base.h"
#include "idiom.h"
#include "opcns.h"

/*  11 <- "b" */
int numDeCharHex (char ch)
{
  if ((ch <= '9') & (ch >= '0'))
    return ch - '0' ;
  else if ((ch <= 'f') & (ch >= 'a'))
   return ch - 'a' + 10 ;
  else
   return -1 ;
}

/*  7 <- "7" */
int numDeCharDec (char ch)
{
  if ((ch <= '9') & (ch >= '0'))
    return ch - '0' ;
  else
   return -1 ;
}

/*  8217 <- "u2019" */
int numDeUstrHex (char * s)
{
  int n1, n2, n3, n4;
  if ((s[0] != 'u') |
      ((n1 = numDeCharHex (s[1])) == -1) |
      ((n2 = numDeCharHex (s[2])) == -1) |
      ((n3 = numDeCharHex (s[3])) == -1) |
      ((n4 = numDeCharHex (s[4])) == -1) ) {
    printf ("numDeUstrHex: error, paramero incorrecto, %5s\n", s);
    return -1;
    }
  return ((((n1 * 16) + n2) * 16 + n3) * 16) + n4 ;
}

int numDeStrDec3 (char * s) /* 3 cifras */
{
  int n0, n1, n2;
  if (((n0 = numDeCharDec (s[0])) == -1) |
      ((n1 = numDeCharDec (s[1])) == -1) |
      ((n2 = numDeCharDec (s[2])) == -1) ) {
    printf ("numDeStrDec: error, paramero incorrecto, %3s\n", s);
    return -1;
    }
  return ((n0 * 10) + n1) * 10 + n2 ;
}

/*  "u2019" <- 8217 */
void ustrHexDeNum (char * s, int n)
{
  sprintf(s, "u%04x", n);
}

void utf8DeNum (char * u, int n)
{
  if (n < 128) {
    u [0] = n ; 
    u [1] = 0;
    }
  else if (n < 2048) {
    u [0] = 192 + n/64 ;
    u [1] = 128 + n%64 ;
    u [2] = 0;
    }
  else if (n < 65575) {
    u [0] = 224 + n/4096 ;
    u [1] = 128 + (n/64)%64 ;
    u [2] = 128 + n%64 ;
    u [3] = 0;
    }
}

int numDeUtf8 (char * u)
{
  int n0, n1, n2;
  n0 = u[0] & 0xFF ;
  if (n0 < 128)
    return n0 ;
  else if (n0 < 192)
    return 0 ;  /* error */
  else if (n0 < 224) {
    n1 = u[1] & 0xFF ;
    return (n0 - 192)*64 + (n1 - 128) ;
    }
  else if (n0 < 240) {
    n1 = u[1] & 0xFF ;
    n2 = u[2] & 0xFF ;
    return (n0 - 224)*4096 + (n1 - 128)*64 + (n2 - 128) ;
    }
  else
    return 0 ;  /* error */
}

int longDeUtf8 (char * u)
{
  int u0 ;
  u0 = 0xFF & u[0] ;
  if (u0 < 127)
    return 1 ;
  else if (u0 < 192)
    return -1 ;
  else if (u0 < 224)
    return 2 ;
  else if (u0 < 240)
    return 3 ;
  else
    return 4 ;
}

/* ------ inicio ------*/

static int nltutf2s;
static char ** tutf2s1, ** tutf2s0;

void leeTutf2salida (char * dir)
{
  if (l_opcion) {
    switch (l_valor_idiom) {
      case idiom_pol:
        leeT2 (dir, "../conv/ts-8859-2.ts",  &tutf2s0, &tutf2s1, & nltutf2s);
	break ;
      case idiom_rus:
        leeT2 (dir, "../conv/ts-8859-5.ts",  &tutf2s0, &tutf2s1, & nltutf2s);
	break ;
      case idiom_fra:
        leeT2 (dir, "../conv/ts-8859-15.ts", &tutf2s0, &tutf2s1, & nltutf2s);
	break ;
	// ingles, espaol, aleman
      default:
        leeT2 (dir, "../conv/ts-8859-1.ts",  &tutf2s0, &tutf2s1, & nltutf2s);
	break ;
      }
    }
  else
    leeT2 (dir, "../conv/ts-8859-1.ts", &tutf2s0, &tutf2s1, & nltutf2s);
}

/* ------ un glifo ------*/

void utf8DeUstr (char * u, char * s)
{
  utf8DeNum ( u, numDeUstrHex(s) );
}

void ustrDeUtf8 (char * s, char * u)
{
  ustrHexDeNum ( s, numDeUtf8(u) ) ;
}

void salidaDeUtf (char * salida, char * nomu)
{
  int n;
  int res;
  char s[6];
  ustrDeUtf8 (s, nomu);
  res = asocT2 (s, salida, tutf2s0, tutf2s1, nltutf2s);
  if (res) {
    if (numDeCharDec(salida[0]) >= 0) {
      n = numDeStrDec3 (salida);
      salida [0] = n;
      salida [1] = 0;
    }
    }
  if (! res) {
/*
    printf ("ocre: cutfAsalida, no se encuentra %s\n", nomu) ;
    exit (1);
*/
    strcpy (salida, nomu);
    }
}

/* ------ varios glifos ------*/

void sutf8DeUstr (char * u, char * s)
{
  int i, j;
  i = 0; j = 0;
  while (s[i] != '\0') {
    utf8DeNum ( &(u[j]), numDeUstrHex( &(s[i])) );
    i += 5 ;
    j += strlen ( &(u[j]));
    }
}

void sustrDeUtf8 (char * s, char * u)
{
  int i, j;
  i = 0; j = 0;
  while (u[i] != '\0') {
    ustrDeUtf8 ( &(s[j]), &(u[i]) );
    i += longDeUtf8 ( &(u[i]) );
    j += 5 ;
    }
}

void ssalidaDeUtf (char * salida, char * nomu)
{
  int i, j;
  i = 0; j = 0;
  while (nomu[i] != '\0') {
    salidaDeUtf ( &(salida[j]), &(nomu[i]) );
    i += longDeUtf8 ( &(nomu[i]) );
    j += strlen ( &(salida[j]) );
    }
}

Boolean esUstr (char * s)
{
  int i, l;
  l = strlen(s);
  if (l%5 != 0)
    return FALSE;
  for (i=0; i<l; i++) {
    if (i%5 == 0) {
      if (s[i] != 'u')
	return FALSE;
      }
    else
      if (((s[i] < '0') || (s[i] > '9')) &&
          ((s[i] < 'a') || (s[i] > 'f')))
	return FALSE;
    }
  return TRUE;
}

